classdef aeRegressionLayer < nnet.layer.RegressionLayer
    % custom regression layer with absolute-error loss.
    
    methods
        function layer = aeRegressionLayer(name)
            % layer = maeRegressionLayer(name) creates a
            % absolute-error regression layer and specifies the layer
            % name.
            
            % Set layer name.
            layer.Name = name;
            
            % Set layer description.
            layer.Description = 'absolute error';
        end
        
        function loss = forwardLoss(layer, Y, T)
            % loss = forwardLoss(layer, Y, T) returns the AE loss between
            % the predictions Y and the training targets T.
            % 6 output
            %Y(:,:,3:6,:)=100*Y(:,:,3:6,:);
            %T(:,:,3:6,:)=100*T(:,:,3:6,:);
            %relError = sum(abs(Y-T)./abs(T),3);
            absError=sum(abs(Y-T),3);
            % mini-batch
            %loss = sum(relError,4);
            loss=sum(absError,4);%/size(T,4);
        end

        function dLdY = backwardLoss(layer, Y, T)
            % dLdY = backwardLoss(layer, Y, T) returns the derivatives of
            % the MAE loss with respect to the predictions Y.
            %dLdY=sign(Y-T)./abs(T);
            dLdY = sign(Y-T);%/size(Y,3)/size(Y,4);
            
        end
    end
end